/*
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 */

/*  History:
 *  18.04.01: now ignores entities for ascii control characters (0-31)
 *  19.04.01: added parse_entities(char *)
 *  03.09.01: added hexadecimal entities
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>      

#include "latin1.h"
#include "text.h"
#include "debug.h"
#include "util.h"
#include "main.h"
#include "microsoft.h"

int entity_number(char *s);
int html_entity(char*);
int latin1(char*);

// ------------------------------------------------

// parse entity in string 
void parse_entity(char *str)
{
	char tmpstr[DEF_STR_LEN];
	int len = strlen(str);

#ifdef proc_debug
	printf("start parse: %s\n", str);
#endif
	
	strcpy(tmpstr, str);

	//	fprintf(stderr, "tmpstr: %s-\n", tmpstr);
	//fprintf(stderr, "tmpstr: %d-\n", tmpstr[len]);
	if (tmpstr[len-1]!=';') 
  {
		tmpstr[len]   = ';';
		tmpstr[len+1] = '\0';
	}
	//	printf("tmpstr: %s\n", tmpstr);

	if ((entity_number(tmpstr))||(html_entity(tmpstr))||(latin1(tmpstr))||(microsoft_entities(tmpstr))) 
	{
		// if true entity was known
		strcpy(str, tmpstr);
		//fprintf(stderr, "Entity String %s\n", str);
	}
	else
	{
		if (errorlevel>=1) { fprintf(stderr, "entity %s unknown\n",tmpstr); }
	}
	
	//	fprintf(stderr, "Entity String %s %s\n", str, tmpstr);
#ifdef proc_debug
	printf("end parse: %s\n", str);
#endif
} // end parse

// ------------------------------------------------

// parses entities in string
void parse_entities(char *s)
{
	int i=0,j=0,k=0;
	char tmp[DEF_STR_LEN];
	char entity[DEF_STR_LEN];
  int len=strlen(s);
	char result[DEF_STR_LEN];
	
	result[0] = '\0';

	//printf("Ganzer String -%s-\n", s);
	
	while(i<=len)
	{
		j=0;
		while((s[i]!='\0') && (s[i]!='&'))
	  {
			tmp[j++] = s[i++];
		}
		tmp[j] = '\0';
		strcat(result, tmp);

		if (s[i]=='&')
		{
		  k=0;
			//entity[0] = '&';
			while((s[i]!='\0') && (s[i]!=';') && (!isspace(s[i])))
	    {
				entity[k++] = s[i++];
			}
			entity[k] = '\0';
			parse_entity(entity);

			strcat(result, entity);
		}
		i++;
	}

	//printf("Result: %s\n", result);

	strcpy(s, result);
} // end parse_entities

// ------------------------------------------------

int entity_number(char *s)
{
  int number;
  char *tmp = s;

  // Numeric entity
  if ((s[0]=='&') && (s[1]=='#'))
  {	   
	// Hex entity
	if (uppercase(s[2])=='X')
	{
	  tmp += 3;
	  number = x2dec(tmp, 16);					 
	} 
	// Decimal entity
	else {
	  tmp += 2;
	  number = atoi(tmp);	
	}

	// ascii printable character 32-127 
	if ((number>=32) && (number<=127)) 
	{ 
	  set_char(s, number);
	  return(1);
	}
	// ansi printable character 160-255
	else if ((number>=160) && (number<=255)) 
	{
	  //fprintf(stderr, "1 sevenbit %d\n", sevenbit);
	  if (sevenbit) { return(0); }
	  set_char(s, number);
	  return(1);
	}
	// ascii control character -> return empty string
	else if ((number >= 0) && (number < 32)) 
	{ 
	  s[0] = '\0';
	  return(1); 
	}
  }
  return(0);
} // end entity_number

// ------------------------------------------------

int html_entity(char *str)
{	
  if (sevenbit)
  {
	// clutch until proper charset conversion is done
	if (strcmp("&nbsp;", str)==0)    { set_char(str, ' '); } // no-break space 
	else if (strcmp("&#160;", str)==0)    { set_char(str, ' '); } // no-break space 
  }

  if (strcmp("&quot;",      str)==0)    { set_char(str, '"'); }
  else if (strcmp("&;",     str)==0)    { set_char(str, '&'); } // for those brain damaged ones
  else if (strcmp("&amp;",  str)==0)    { set_char(str, '&'); }
  else if (strcmp("&gt;",   str)==0)    { set_char(str, '>'); }
  else if (strcmp("&lt;",   str)==0)    { set_char(str, '<'); }
  else { return(0); } // found no html entity
  return(1); // found html entity 
} // end html_entity

// ------------------------------------------------

// returns true if str is a known entity and changes str to the printout
int latin1(char *str)
{
  //fprintf(stderr, "2 sevenbit %d\n", sevenbit);
  if (sevenbit) { return(0); } // leave char>127 alone

  if (strcmp("&nbsp;", str)==0)         { set_char(str, 160); } // no-break space 
  else if (strcmp("&iexcl", str)==0)    { set_char(str, 161); } // inverted exclamation mark 
  else if (strcmp("&cent;", str)==0)    { set_char(str, 162); } // cent sign 
  else if (strcmp("&pound;", str)==0)   { set_char(str, 163); } // pound sterling sign 
  else if (strcmp("&curren;", str)==0)  { set_char(str, 164); } // general currency sign 
  else if (strcmp("&yen;", str)==0)     { set_char(str, 165); } // yen sign 
  else if (strcmp("&brvbar;", str)==0)  { set_char(str, 166); } // broken (vertical) bar 
  else if (strcmp("&sect;", str)==0)    { set_char(str, 167); } // section sign 
  else if (strcmp("&uml;", str)==0)     { set_char(str, 168); } // umlaut (dieresis) 
  else if (strcmp("&copy;", str)==0)    { set_char(str, 169); } // copyright sign 
  else if (strcmp("&ordf;", str)==0)    { set_char(str, 170); } // ordinal indicator, feminine 
  else if (strcmp("&laquo;", str)==0)   { set_char(str, 171); } // angle quotation mark, left 
  else if (strcmp("&not;", str)==0)     { set_char(str, 172); } // not sign 
  else if (strcmp("&shy;", str)==0)     { set_char(str, 173); } // soft hyphen 
  else if (strcmp("&reg;", str)==0)     { set_char(str, 174); } // registered sign 
  else if (strcmp("&macr;", str)==0)    { set_char(str, 175); } // macron 
  else if (strcmp("&deg;", str)==0)     { set_char(str, 176); } // degree sign 
  else if (strcmp("&plusmn;", str)==0)  { set_char(str, 177); } // plus-or-minus sign 
  else if (strcmp("&sup2;", str)==0)    { set_char(str, 178); } // superscript two 
  else if (strcmp("&sup3;", str)==0)    { set_char(str, 179); } // superscript three 
  else if (strcmp("&acute;", str)==0)   { set_char(str, 180); } // acute accent 
  else if (strcmp("&micro;", str)==0)   { set_char(str, 181); } // micro sign 
  else if (strcmp("&para;", str)==0)    { set_char(str, 182); } // pilcrow (paragraph sign) 
  else if (strcmp("&middot;", str)==0)  { set_char(str, 183); } // middle dot 
  else if (strcmp("&cedil;", str)==0)   { set_char(str, 184); } // cedilla 
  else if (strcmp("&sup1;", str)==0)    { set_char(str, 185); } // superscript one 
  else if (strcmp("&ordm;", str)==0)    { set_char(str, 186); } // ordinal indicator, masculine 
  else if (strcmp("&raquo;", str)==0)   { set_char(str, 187); } // angle quotation mark, right 
  else if (strcmp("&frac14;", str)==0)  { set_char(str, 188); } // fraction one-quarter 
  else if (strcmp("&frac12;", str)==0)  { set_char(str, 189); } // fraction one-half 
  else if (strcmp("&frac34;", str)==0)  { set_char(str, 190); } // fraction three-quarters 
  else if (strcmp("&iquest;", str)==0)  { set_char(str, 191); } // inverted question mark 
  else if (strcmp("&Agrave;", str)==0)  { set_char(str, 192); } // capital A, grave accent 
  else if (strcmp("&Aacute;", str)==0)  { set_char(str, 193); } // capital A, acute accent 
  else if (strcmp("&Acirc;", str)==0)   { set_char(str, 194); } // capital A, circumflex accent 
  else if (strcmp("&Atilde;", str)==0)  { set_char(str, 195); } // capital A, tilde 
  else if (strcmp("&Auml;", str)==0)    { set_char(str, 196); } // capital A, dieresis or umlaut mark 
  else if (strcmp("&Aring;", str)==0)   { set_char(str, 197); } // capital A, ring 
  else if (strcmp("&AElig;", str)==0)   { set_char(str, 198); } // capital AE diphthong (ligature) 
  else if (strcmp("&Ccedil;", str)==0)  { set_char(str, 199); } // capital C, cedilla 
  else if (strcmp("&Egrave;", str)==0)  { set_char(str, 200); } // capital E, grave accent 
  else if (strcmp("&Eacute;", str)==0)  { set_char(str, 201); } // capital E, acute accent 
  else if (strcmp("&Ecirc;", str)==0)   { set_char(str, 202); } // capital E, circumflex accent 
  else if (strcmp("&Euml;", str)==0)    { set_char(str, 203); } // capital E, dieresis or umlaut mark 
  else if (strcmp("&Igrave;", str)==0)  { set_char(str, 204); } // capital I, grave accent 
  else if (strcmp("&Iacute;", str)==0)  { set_char(str, 205); } // capital I, acute accent 
  else if (strcmp("&Icirc;", str)==0)   { set_char(str, 206); } // capital I, circumflex accent 
  else if (strcmp("&Iuml;", str)==0)    { set_char(str, 207); } // capital I, dieresis or umlaut mark 
  else if (strcmp("&ETH;", str)==0)     { set_char(str, 208); } // capital Eth, Icelandic 
  else if (strcmp("&Ntilde;", str)==0)  { set_char(str, 209); } // capital N, tilde 
  else if (strcmp("&Ograve;", str)==0)  { set_char(str, 210); } // capital O, grave accent 
  else if (strcmp("&Oacute;", str)==0)  { set_char(str, 211); } // capital O, acute accent 
  else if (strcmp("&Ocirc;", str)==0)   { set_char(str, 212); } // capital O, circumflex accent 
  else if (strcmp("&Otilde;", str)==0)  { set_char(str, 213); } // capital O, tilde 
  else if (strcmp("&Ouml;", str)==0)    { set_char(str, 214); } // capital O, dieresis or umlaut mark 
  else if (strcmp("&times;", str)==0)   { set_char(str, 215); } // multiply sign 
  else if (strcmp("&Oslash;", str)==0)  { set_char(str, 216); } // capital O, slash 
  else if (strcmp("&Ugrave;", str)==0)  { set_char(str, 217); } // capital U, grave accent 
  else if (strcmp("&Uacute;", str)==0)  { set_char(str, 218); } // capital U, acute accent 
  else if (strcmp("&Ucirc;", str)==0)   { set_char(str, 219); } // capital U, circumflex accent 
  else if (strcmp("&Uuml;", str)==0)    { set_char(str, 220); } // capital U, dieresis or umlaut mark 
  else if (strcmp("&Yacute;", str)==0)  { set_char(str, 221); } // capital Y, acute accent 
  else if (strcmp("&THORN;", str)==0)   { set_char(str, 222); } // capital THORN, Icelandic 
  else if (strcmp("&szlig;", str)==0)   { set_char(str, 223); } // small sharp s, German (sz ligature) 
  else if (strcmp("&agrave;", str)==0)  { set_char(str, 224); } // small a, grave accent 
  else if (strcmp("&aacute;", str)==0)  { set_char(str, 225); } // small a, acute accent 
  else if (strcmp("&acirc;", str)==0)   { set_char(str, 226); } // small a, circumflex accent 
  else if (strcmp("&atilde;", str)==0)  { set_char(str, 227); } // small a, tilde 
  else if (strcmp("&auml;", str)==0)    { set_char(str, 228); } // small a, dieresis or umlaut mark 
  else if (strcmp("&aring;", str)==0)   { set_char(str, 229); } // small a, ring 
  else if (strcmp("&aelig;", str)==0)   { set_char(str, 230); } // small ae diphthong (ligature) 
  else if (strcmp("&ccedil;", str)==0)  { set_char(str, 231); } // small c, cedilla 
  else if (strcmp("&egrave;", str)==0)  { set_char(str, 232); } // small e, grave accent 
  else if (strcmp("&eacute;", str)==0)  { set_char(str, 233); } // small e, acute accent 
  else if (strcmp("&ecirc;", str)==0)   { set_char(str, 234); } // small e, circumflex accent 
  else if (strcmp("&euml;", str)==0)    { set_char(str, 235); } // small e, dieresis or umlaut mark 
  else if (strcmp("&igrave;", str)==0)  { set_char(str, 236); } // small i, grave accent 
  else if (strcmp("&iacute;", str)==0)  { set_char(str, 237); } // small i, acute accent 
  else if (strcmp("&icirc;", str)==0)   { set_char(str, 238); } // small i, circumflex accent 
  else if (strcmp("&iuml;", str)==0)    { set_char(str, 239); } // small i, dieresis or umlaut mark 
  else if (strcmp("&eth;", str)==0)     { set_char(str, 240); } // small eth, Icelandic 
  else if (strcmp("&ntilde;", str)==0)  { set_char(str, 241); } // small n, tilde 
  else if (strcmp("&ograve;", str)==0)  { set_char(str, 242); } // small o, grave accent 
  else if (strcmp("&oacute;", str)==0)  { set_char(str, 243); } // small o, acute accent 
  else if (strcmp("&ocirc;", str)==0)   { set_char(str, 244); } // small o, circumflex accent 
  else if (strcmp("&otilde;", str)==0)  { set_char(str, 245); } // small o, tilde 
  else if (strcmp("&ouml;", str)==0)    { set_char(str, 246); } // small o, dieresis or umlaut mark 
  else if (strcmp("&divide;", str)==0)  { set_char(str, 247); } // divide sign 
  else if (strcmp("&oslash;", str)==0)  { set_char(str, 248); } // small o, slash 
  else if (strcmp("&ugrave;", str)==0)  { set_char(str, 249); } // small u, grave accent 
  else if (strcmp("&uacute;", str)==0)  { set_char(str, 250); } // small u, acute accent 
  else if (strcmp("&ucirc;", str)==0)   { set_char(str, 251); } // small u, circumflex accent 
  else if (strcmp("&uuml;", str)==0)    { set_char(str, 252); } // small u, dieresis or umlaut mark 
  else if (strcmp("&yacute;", str)==0)  { set_char(str, 253); } // small y, acute accent 
  else if (strcmp("&thorn;", str)==0)   { set_char(str, 254); } // small thorn, Icelandic 
  else if (strcmp("&yuml;", str)==0)    { set_char(str, 255); } // small y, dieresis or umlaut mark 
  
  else { return(0); } // found no latin1 entity
  return(1); // found latin1 entity 
} // end latin1
