/*
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 * 
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 */

#include <string.h>
#include <stdio.h>

#include "debug.h"

#include "text.h"
#include "main.h"
#include "util.h"

// ------------------------------------------------

int microsoft_entities(char *s)
{
  if (!microsoft) { return(0); }
  // Euro
  else if (strcmp("&#128;", s)==0)    { strcpy(s, "EUR"); }
  else if (strcmp("&euro;", s)==0)    { strcpy(s, "EUR"); }
  else if (strcmp("&#8364;", s)==0)   { strcpy(s, "EUR"); }
  
  // Single Low-9 Quotation Mark
  else if (strcmp("&#130;", s)==0)    { set_char(s, ','); }
  else if (strcmp("&sbquo;", s)==0)   { set_char(s, ','); }
  else if (strcmp("&#8218;", s)==0)   { set_char(s, ','); }

  else if (strcmp("&#131;", s)==0)    { set_char(s, 'f'); } // Latin Small Letter F With Hook
  else if (strcmp("&fnof;", s)==0)    { set_char(s, 'f'); } // Latin Small Letter F With Hook
  else if (strcmp("&#402;", s)==0)    { set_char(s, 'f'); } // Latin Small Letter F With Hook

  // Double Low-9 Quotation Mark
  else if (strcmp("&#132;", s)==0)    { strcpy(s, "\""); }
  else if (strcmp("&bdquo;", s)==0)   { strcpy(s, "\""); }
  else if (strcmp("&#8222;", s)==0)   { strcpy(s, "\""); }

  else if (strcmp("&#133;", s)==0)    { strcpy(s, "... "); } // Horizontal Ellipsis
  else if (strcmp("&hellip;", s)==0)  { strcpy(s, "... "); } // Horizontal Ellipsis
  else if (strcmp("&#8230;", s)==0)   { strcpy(s, "... "); } // Horizontal Ellipsis

  // Dagger
  else if (strcmp("&#134;", s)==0)    { strcpy(s, "/-"); }   
  else if (strcmp("&dagger;", s)==0)  { strcpy(s, "/-"); }
  else if (strcmp("&#8224;", s)==0)   { strcpy(s, "/-"); }

  // Double Dagger
  else if (strcmp("&#135;", s)==0)    { strcpy(s, "/="); }
  else if (strcmp("&Dagger;", s)==0)  { strcpy(s, "/="); }
  else if (strcmp("&#8225;", s)==0)   { strcpy(s, "/="); }
  
  // Modifier Letter Circumflex Accent
  else if (strcmp("&#136;", s)==0)    { set_char(s, '^'); }
  else if (strcmp("&circ;", s)==0)    { set_char(s, '^'); }
  else if (strcmp("&#710;", s)==0)    { set_char(s, '^'); }

  // Per Mille Sign
  else if (strcmp("&#137;", s)==0)    { strcpy(s, "O/oo"); }
  else if (strcmp("&permil;", s)==0)  { strcpy(s, "O/oo"); }
  else if (strcmp("&#8240;", s)==0)   { strcpy(s, "O/oo"); } 

  // Latin Capital Letter S With Caron
  else if (strcmp("&#138;", s)==0)    { set_char(s, 'S'); }
  else if (strcmp("&Scaron;", s)==0)  { set_char(s, 'S'); }
  else if (strcmp("&#352;", s)==0)    { set_char(s, 'S'); }

  // Single Left-Pointing Angle Quotation Mark
  else if (strcmp("&#139;", s)==0)    { set_char(s, '<'); }
  else if (strcmp("&lsaquo;", s)==0)  { set_char(s, '<'); }
  else if (strcmp("&#8249;", s)==0)   { set_char(s, '<'); }

  // Latin Capital Ligature OE
  else if (strcmp("&#140;", s)==0)    { strcpy(s, "OE"); }  
  else if (strcmp("&OElig;", s)==0)   { strcpy(s, "OE"); }   
  else if (strcmp("&#338;", s)==0)    { strcpy(s, "OE"); }  

  // Z\/
  else if (strcmp("&#142;", s)==0)    { set_char(s, 'Z'); }
  else if (strcmp("&#381;", s)==0)    { set_char(s, 'Z'); }

  // Left Single Quotation Mark
  else if (strcmp("&#145;", s)==0)    { set_char(s, '`'); }
  else if (strcmp("&lsquo;", s)==0)   { set_char(s, '`'); }
  else if (strcmp("&#8216;", s)==0)   { set_char(s, '`'); }

  // Right Single Quotation Mark
  else if (strcmp("&#146;", s)==0)    { set_char(s, '\''); }
  else if (strcmp("&rsquo;", s)==0)   { set_char(s, '\''); }
  else if (strcmp("&#8217;", s)==0)   { set_char(s, '\''); }

  // Left Double Quotation Mark
  else if (strcmp("&#147;", s)==0)    { set_char(s, '"'); }
  else if (strcmp("&ldquo;", s)==0)   { set_char(s, '"'); }
  else if (strcmp("&#8220;", s)==0)   { set_char(s, '"'); }

  // Right Double Quotation Mark
  else if (strcmp("&#148;", s)==0)    { set_char(s, '"'); }
  else if (strcmp("&rdquo;", s)==0)   { set_char(s, '"'); }
  else if (strcmp("&#8221;", s)==0)   { set_char(s, '"'); }

  // Bullet
  else if (strcmp("&#149;", s)==0)    { set_char(s, '*'); }
  else if (strcmp("&bull;", s)==0)    { set_char(s, '*'); }
  else if (strcmp("&#8226;", s)==0)   { set_char(s, '*'); }
  
  // En Dash
  else if (strcmp("&#150;", s)==0)    { set_char(s, '-'); }
  else if (strcmp("&ndash;", s)==0)   { set_char(s, '-'); }
  else if (strcmp("&#8211;", s)==0)   { set_char(s, '-'); }

  // Em Dash
  else if (strcmp("&#151;", s)==0)    { strcpy(s, "--"); }
  else if (strcmp("&mdash;", s)==0)   { strcpy(s, "--"); }
  else if (strcmp("&#8212;", s)==0)   { strcpy(s, "--"); }
  
  // Small Tilde
  else if (strcmp("&#152;", s)==0)    { set_char(s, '~'); }
  else if (strcmp("&tilde;", s)==0)   { set_char(s, '~'); }
  else if (strcmp("&#732;", s)==0)    { set_char(s, '~'); } 

  // Trade Mark Sign
  else if (strcmp("&#153;", s)==0)    { strcpy(s, "[tm]"); }
  else if (strcmp("&trade;", s)==0)   { strcpy(s, "[tm]"); }
  else if (strcmp("&#8482;", s)==0)   { strcpy(s, "[tm]"); }

  // Latin Small Letter S With Caron
  else if (strcmp("&#154;", s)==0)    { set_char(s, 's'); }
  else if (strcmp("&scaron;", s)==0)  { set_char(s, 's'); }
  else if (strcmp("&#353;", s)==0)    { set_char(s, 's'); }

  // Single Right-Pointing Angle Quotation Mark
  else if (strcmp("&#155;", s)==0)    { set_char(s, '>'); }
  else if (strcmp("&rsaquo;", s)==0)  { set_char(s, '>'); }
  else if (strcmp("&#8250;", s)==0)   { set_char(s, '>'); }

  // Latin Small Ligature OE
  else if (strcmp("&#156;", s)==0)    { strcpy(s, "oe"); }
  else if (strcmp("&oelig;", s)==0)   { strcpy(s, "oe"); }
  else if (strcmp("&#339;", s)==0)    { strcpy(s, "oe"); }

  // z\/
  else if (strcmp("&#158;", s)==0)    { set_char(s, 'z'); }
  else if (strcmp("&#382;", s)==0)    { set_char(s, 'z'); }

  // Latin Capital Letter Y With Diaeresis 
  else if (strcmp("&#159;", s)==0)    { set_char(s, 'Y'); }
  else if (strcmp("&Yuml;", s)==0)    { set_char(s, 'Y'); }
  else if (strcmp("&#376;", s)==0)    { set_char(s, 'Y'); }
	
  else { return(0); }

  return(1); // Microsoft entity found
} // end microsoft_entities

// ------------------------------------------------

void microsoft_character(int c)
{
  //fprintf(stderr, "3 sevenbit %d\n", sevenbit);
  if (sevenbit) 
  { 
	wort_plus_ch(c);
	return;
  }

  switch (c)
  {
	// Microsoft...
  case 0x80: /* MICROSOFT EURO */
	wort_plus_string("EUR"); break;
	
  case 0x82: /* SINGLE LOW-9 QUOTATION MARK */
	wort_plus_ch(','); break;
  case 0x83: /* Latin Small Letter F With Hook */
	wort_plus_ch('f'); break;
  case 0x84: /* Double Low-9 Quotation Mark */
	wort_plus_string("\""); break;
  case 0x85: /* HORIZONTAL ELLIPSIS */
	wort_plus_string("..."); break;
  case 0x86: /* Dagger */
	wort_plus_string("/-"); break;
  case 0x87: /* Double Dagger */
	wort_plus_string("/="); break;
  case 0x88: /* Modifier Letter Circumflex Accent */
	wort_plus_ch('^'); break;
  case 0x89: /* Per Mille Sign */
	wort_plus_string("O/oo"); break;
  case 0x8a: /* Latin Capital Letter S With Caron */
	wort_plus_ch('S'); break;
  case 0x8b: /*  Single Left-Pointing Angle Quotation Mark */
	wort_plus_ch('<'); break;
  case 0x8c: /* Latin Capital Ligature OE */
	wort_plus_string("OE"); break;
	
  case 0x8e: /* Z\/ */
	wort_plus_ch('Z'); break;
	
  case 0x91: /* LEFT SINGLE QUOTATION MARK */
	wort_plus_ch('`'); break;
  case 0x92: /* RIGHT SINGLE QUOTATION MARK */
	wort_plus_ch('\''); break;
  case 0x93: /* LEFT DOUBLE QUOTATION MARK */
	wort_plus_ch('\"'); break;
  case 0x94: /* RIGHT DOUBLE QUOTATION MARK */
	wort_plus_ch('\"'); break;
  case 0x95: /* BULLET */
	wort_plus_ch('*'); break;
  case 0x96: /* EN DASH */
	wort_plus_ch('-'); break;
  case 0x97: /* EM DASH */
	wort_plus_string("--"); break;
  case 0x98: /* SMALL TILDE */
	wort_plus_ch('~'); break;
  case 0x99: /* TRADE MARK SIGN */
	wort_plus_string("[tm]"); break;
  case 0x9a: /* LATIN SMALL LETTER S WITH CARON */
	wort_plus_ch('s'); break;
  case 0x9b: /* SINGLE RIGHT-POINTING ANGLE QUOTATION MARK */
	wort_plus_ch('>'); break;
  case 0x9c: /* LATIN SMALL LIGATURE OE */
	wort_plus_string("oe"); break;
	
  case 0x9e: /* z\/ */
	wort_plus_ch('z'); break;
  case 0x9f: /* LATIN CAPITAL LETTER Y WITH DIAERESIS */
	wort_plus_ch('Y'); break;
  }
} // end microsoft_character
